<?php defined('SYSPATH') or die('No direct script access.');

class SngTemplate {
	private $m_xml;

	public $customer;
	public $product;
	public $configuration;

	public $description;
	public $target_description;

	public function __construct(SimpleXMLElement $e) {
		$id = $e->general->id->attributes();
		$this->customer = (string)$id["customer"];
		$this->product = (string)$id["product"];
		$this->configuration = (string)$id["configuration"];
		$this->target_description = (string)$e->targets->description;
		$this->description = trim($e->general->description);
		$this->m_xml = $e;
	}

	public function fields() {
		$fields = array();
		$xfields = $this->m_xml->xpath("//parameters/parameter");
		foreach ($xfields as $xfield) {
			$fields[] = new SngTemplateField($xfield);
		}
		return $fields;
	}

	public function validate($values) {
		$fields = $this->fields();
		$errors = array();

		foreach ($fields as $field) {
			$value = isset($values[$field->field]) ? $values[$field->field] : "";
			if (true !== ($result = $field->validate($value))) {
                $errors = array_merge($errors, $result);
			}
		}

		if (count($errors) == 0) {
			return true;
		} else {
			return $errors;
		}
	}

	public function targets() {
		$xtargets = $this->m_xml->xpath("//targets/file");
		$targets = array();
		foreach($xtargets as $xtarget) {
			$attribs = $xtarget->attributes();
			$targets[] = (string)$attribs['name'];
		}

		return $targets;
	}

	public function render_target($name, $input_values) {
		$xdata = $this->m_xml->xpath("//targets/file[@name='$name']");
		$data = (string)$xdata[0]->data;

		/* Prepare substitution values */
		$values = array();
		foreach($this->fields() as $field) {
			if ($field->type != "generated" && array_key_exists($field->field, $input_values)) {
				if ($field->validate($input_values[$field->field])) {
					$values[$field->field] = $input_values[$field->field];
                } else {
					return FALSE; /* Validation failed */
                }
            }
		}

        $replace = array();
        foreach ($values as $key => $value) {
            $replace['$$'.$key.'$$'] = $value;
        }

        /* Run generated content */
        foreach($this->fields() as $field) {
            if ($field->type == "generated") {
                /* Parse the "rules" section into function calls, separated by ;" */
                $code = strtr($field->code, $replace);
                $code = "\$value = ".$code .";";

                eval($code);

                $replace['$$'.$field->field.'$$'] = $values[$field->field] = $value;
            }
        }


	/* Add generated values */
	$replace['$$sacs_template_name$$'] = $this->customer.'-'.$this->product.'-'.$this->configuration;
	$replace['$$sacs_generate_date$$'] = date("d/j/Y H:i:s");

	/* Run substitution rules for each field */


        $data = strtr($data, $replace);

		return $data;
	}
}

