<?php defined('SYSPATH') or die('No direct script access.');


class SngTemplateField {
  public $field;
  public $label;
  public $type;
  public $rules;
  public $code;
  public $default;
  public $help;
  public $options = array();

  public function __construct(SimpleXMLElement $e) {
    $attribs= $e->attributes();
    $this->field = (string)$attribs["field"];
    $this->label = (string)$attribs["label"];
    $this->type = (string)$attribs["type"];
    $this->rules = (string)$attribs["rules"];
    $this->code = (string)$attribs["code"];
    $this->default = (string)$attribs["default"];
    $this->help = (string)trim(implode('',$e->xpath("description")));
    $options = $e->xpath("options/option");
    if($options){
      foreach($options as $option){
        $attr = $option->attributes();
        $this->options[(string)$attr["value"]] = (string)$attr["label"];
      }
    }
  }

  public static function parse_rule($rule)
  {
    $param = "";
    /* Check for a parameter */
    if (($pos = strpos($rule, "[")) !== FALSE) {
      $endpos = strpos($rule, "]", $pos);
      if ($endpos === FALSE) {
        throw new Kohana_Exception("Syntax error in rule name: ".$rule);
      }
      $param = substr($rule, $pos+1, $endpos - $pos-1);
      $rule = substr($rule, 0, $pos);
    }
    if (strlen($param) > 0) {
      return array($rule, $param);
    } else {
      return array($rule, null);
    }
  }

  public function validate($value)
  {
    if (empty($this->rules)) {
      return true;
    }

    $rules = explode("|", $this->rules);
    foreach($rules as $rule) {
      $param = "";

      list($rule, $param) = SngTemplateField::parse_rule($rule);

      if (strlen($param) > 0) { /* empty() considers "0" to be an empty string */
        $r = call_user_func_array(array("Valid", $rule), array($value, $param));
      } else {
        $r = call_user_func_array(array("Valid", $rule), array($value));
      }

      if ($r !== true) {
        return array($this->field => $rule);
      }
    }

    return true;
  }
}
